!BM--------------------------
module carint

contains
  !BS-------------------------------------------------------
  subroutine cartoint(atom_types,xyz,N,output)
  use stringlengths
  implicit none
  integer, intent(in) :: N
  character*2, intent(in) :: atom_types(N)
  double precision, intent(in) :: xyz(N,3)
  character(LEN=*) :: output
  double precision ::  degree, geo(N,3), iz(N)
  integer :: na(N), nb(N), nc(N)
  integer :: i, j
  character*4 :: test
  !***********************************************************************
  ! 
  ! CARTOINT CONVERTS AN ARRAY OF CARTESIAN COORDINATES XYZ TO INTERNAL
  !          COORDINATES, AND PRINTS THEM TO A GIVEN FILENAME "output"
  !          IN A FORMAT SUITED FOR THE QUANTUM CHEMISTRY PROGRAM 
  !          C-FOUR. THE FORMATTING IS SPECIFIED SUCH THAT C-FOUR 
  !          CALCULATES A GEOMETRY OPTIMIZATION OF THE ENTIRE SYSTEM.
  !          
  !          WHEN XYZINT FINISHES, ATOMS I WILL HAVE THEIR CLOSEST 
  !          NEIGHBOUR IN NA(I), SECOND CLOSEST IN NB(I) AND THIRD IN 
  !          NC(I). THE RADIUS/DISTANCE FROM ATOM I TO ITS CLOSEST NA(I)
  !          IS GEO(I,1). THE ANGLE BETWEEN ATOMS I, NA(I) AND NB(I) IS
  !          IN GEO(I,2). THE DIHEDRAL ANGLE FROM ATOM I VIA NA(I), NB(I)
  !          AND NC(I) IS IN GEO(I,3).
  !          
  !          FORMAT OF COORDINATE-FILE/SYSTEM IS THUS:
  !          ATOM    NA(I)    GEO(I,1)    NB(I)   GEO(I,2)  NC  GEO(I,3)
  !          
  !***********************************************************************
  
  degree = 57.29578d0 ! Degrees, not radians
  call xyzint(xyz,N,na,nb,nc,degree,geo)
  
  ! Write the interal coordinates to file with right formatting
  
  open(unit=55, file=output, status='new')
  
  write(55,'(a1)'), atom_types(1)
  write(55,'(a1,i2,a2,a1,a1)'), atom_types(2), na(2), "r","2","*"
  write(55,'(a1,i2,a2,a1,a1,i2,a2,a1,a1)'), atom_types(3), na(3), "r","3","*", nb(3), "a","3","*"
  
  do i = 4,N
    if (geo(i,3) .gt. 90.d0) then
      geo(i,3) = 360.d0 - geo(i,3) ! This is a very poor solution, I think...
    end if 
    if (i .lt. 10) then
      write(55,'(a1,i2,a2,i1,a2,i1,a2,i1,a2,i1,a2,i1,a1)'),atom_types(i),na(i),"r",i,"* ",nb(i),"a",i,"* ",nc(i),"d",i,"*"
    elseif (i .ge. 10 .and. i .lt. 100 .and. na(i) .lt. 10) then
      write(55,'(a1,i2,a2,i2,a2,i1,a2,i2,a2,i1,a2,i2,a1)'),atom_types(i),na(i),"r",i,"* ",nb(i),"a",i,"* ",nc(i),"d",i,"*"
    end if
  end do
  
  write(55,*) " "
  
  write(55,'(a1,i1,a1,f6.4)'),"r",2,"=",geo(2,1)
  write(55,'(a1,i1,a1,f6.4)'),"r",3,"=",geo(3,1)
  write(55,'(a1,i1,a1,f6.2)'),"a",3,"=",geo(3,2)
  
  do i = 4,N
    if (i .lt. 10) then
      write(55,'(a1,i1,a1,f6.4)'),"r",i,"=",geo(i,1)
      write(55,'(a1,i1,a1,f6.2)'),"a",i,"=",geo(i,2)
      write(55,'(a1,i1,a1,f6.2)'),"d",i,"=",geo(i,3)
    elseif (i .ge. 10 .and. i .lt. 100) then
      write(55,'(a1,i2,a1,f6.4)'),"r",i,"=",geo(i,1)
      write(55,'(a1,i2,a1,f6.2)'),"a",i,"=",geo(i,2)
      write(55,'(a1,i2,a1,f6.2)'),"d",i,"=",geo(i,3)
    end if
  end do
  
  close(unit=55)
  
  end subroutine cartoint
  !ES-------------------------------------------------------
  
  !BS-------------------------------------------------------
  subroutine xyzint(xyz,N,na,nb,nc,degree,geo)
  implicit none
  integer, intent(in) :: N
  double precision, intent(in) :: xyz(N,3)
  double precision, intent(in) :: degree
  double precision :: geo(N,3), iz(N), rvec(3), rdist, maxr
  integer :: na(N), nb(N), nc(N), i,j,k
  !***********************************************************************
  !
  ! XYZINT WORKS OUT THE INTERNAL COORDINATES OF A MOLECULE.
  !        THE "RULES" FOR THE CONNECTIVITY ARE AS FOLLOWS:
  !        ATOM I IS DEFINED AS BEING AT A DISTANCE FROM THE NEAREST
  !        ATOM J, ATOM J ALREADY HAVING BEEN DEFINED.
  !        ATOM I MAKES AN ANGLE WITH ATOM J AND THE ATOM K, WHICH HAS
  !        ALREADY BEEN DEFINED, AND IS THE NEAREST ATOM TO J
  !        ATOM I MAKES A DIHEDRAL ANGLE WITH ATOMS J, K, AND L. L HAVING
  !        BEEN DEFINED AND IS THE NEAREST ATOM TO K
  !
  !        NOTE THAT GEO AND XYZ MUST NOT BE THE SAME IN THE CALL.
  !
  !   ON INPUT XYZ    = CARTESIAN ARRAY OF NUMAT ATOMS
  !            DEGREE = 1 IF ANGLES ARE TO BE IN RADIANS
  !            DEGREE = 57.29578 IF ANGLES ARE TO BE IN DEGREES
  !
  !***********************************************************************
  
  do i = 1,N
    na(i) = 2
    nb(i) = 3
    nc(i) = 4
    if(i .eq. 1) then 
      go to 20
    end if
    maxr = 100.0
    do j = 1,i-1
      rvec(:) = (xyz(i,:)-xyz(j,:))
      rdist = sqrt(dot_product(rvec,rvec))
      if (rdist .lt. maxr .and. na(j) .ne. j .and. nb(j) .ne. j) then
        maxr = rdist
        k = j
      end if
      
      ! Atom i is nearest to atom k
      na(i) = k
      
      ! FIND ANY ATOM TO RELATE TO NA(I)
      if (i .gt. 2) then 
        nb(i) = na(k)
      end if
      if (i .gt. 3) then
        nc(i) = nb(k)
      end if
      
    end do
    
    20 continue
    
  end do
  
  na(1) = 0
  nb(1:2) = 0
  nc(1:3) = 0
  
  call xyzgeo(xyz,N,na,nb,nc,degree,geo)
  
  end subroutine xyzint
  !ES-------------------------------------------------------
  
  !BS-------------------------------------------------------
  subroutine xyzgeo(xyz,N,na,nb,nc,degree,geo)
  implicit none
  integer, intent(in) :: N
  double precision, intent(in) :: xyz(N,3), degree
  integer, intent(in) :: na(N), nb(N), nc(N)
  double precision :: geo(N,3), rvec(3)
  integer :: i,j,k,l
  !***********************************************************************
  !
  !   XYZGEO CONVERTS COORDINATES FROM CARTESIAN TO INTERNAL.
  !
  !     ON INPUT XYZ  = ARRAY OF CARTESIAN COORDINATES
  !              N    = NUMBER OF ATOMS
  !              NA   = NUMBERS OF ATOM TO WHICH ATOMS ARE RELATED
  !                     BY DISTANCE
  !              NB   = NUMBERS OF ATOM TO WHICH ATOMS ARE RELATED
  !                     BY ANGLE
  !              NC   = NUMBERS OF ATOM TO WHICH ATOMS ARE RELATED
  !                     BY DIHEDRAL
  !
  !    ON OUTPUT GEO  = INTERNAL COORDINATES IN ANGSTROMS, RADIANS,
  !                     AND RADIANS
  !
  !***********************************************************************
  
  do i = 2,N
    j = na(i)
    k = nb(i)
    l = nc(i)
    rvec(:) = xyz(i,:) - xyz(j,:)
    geo(i,1) = sqrt(dot_product(rvec,rvec))
    if (i .ge. 3) then
      call bangle(xyz,N,i,j,k,geo(i,2))
      geo(i,2)=geo(i,2)*degree
    end if
    
    if (i .ge. 4) then
      call dihed(xyz,N,i,j,k,l,geo(i,3))
      geo(i,3) = geo(i,3)*degree
    end if
    
  end do
  
  geo(1,1:3) = 0.d0
  geo(2,2:3) = 0.d0
  geo(3,3)   = 0.d0
  
  end subroutine xyzgeo
  !ES-------------------------------------------------------
  
  !BS-------------------------------------------------------
  subroutine bangle(xyz,N,i,j,k,angle)
  implicit none
  integer, intent(in) :: N, i, j, k
  double precision, intent(in) ::xyz(N,3)
  double precision :: angle
  double precision :: d2ij, d2ik, d2jk, xy, temp
  !*******************************************************************
  !  
  !  Bangle calculates the angle between atoms i,j and k. the 
  !    cartesian coordinates are in xyz.
  !  
  !*******************************************************************
  
  d2ij = (xyz(i,1)-xyz(j,1))**2 + (xyz(i,2)-xyz(j,2))**2 + (xyz(i,3)-xyz(j,3))**2
  d2jk = (xyz(j,1)-xyz(k,1))**2 + (xyz(j,2)-xyz(k,2))**2 + (xyz(j,3)-xyz(k,3))**2
  d2ik = (xyz(i,1)-xyz(k,1))**2 + (xyz(i,2)-xyz(k,2))**2 + (xyz(i,3)-xyz(k,3))**2
  
  xy = sqrt(d2ij*d2jk)
  
  temp = 0.5d0*(d2ij+d2jk-d2ik)/xy

  if (temp .gt. 1.0d0) then
    temp = 1.0d0
  end if
  if (temp .lt. -1.0d0) then
    temp = -1.0d0
  end if
  
  angle = acos(temp)
  
  end subroutine bangle
  !ES-------------------------------------------------------
  
  !BS-------------------------------------------------------
  subroutine dihed(xyz,N,i,j,k,l,angle)
  implicit none
  integer, intent(in) :: N, i, j, k, l
  double precision, intent(in) :: xyz(N,3)
  double precision :: v1(3),v2(3),v3(3),u1(3),u2(3),theta, angle
  !*********************************************************************
  !
  !      DIHED CALCULATES THE DIHEDRAL ANGLE BETWEEN ATOMS I, J, K,
  !            AND L. (ANGLE BETWEEN PLANES IJK AND JKL) THE CARTESIAN 
  !            COORDINATES OF THESE ATOMS ARE IN ARRAY XYZ.
  !                                                           
  !                        L                                  
  !                       /                                   
  !             J--------K                                    
  !            /                                              
  !           I                                               
  !                                                           
  !*********************************************************************
  
  v1(:) = xyz(j,:) - xyz(i,:)
  v2(:) = xyz(k,:) - xyz(j,:)
  v3(:) = xyz(l,:) - xyz(k,:)
  
  u1(1) = v1(2)*v2(3) - v1(3)*v2(2)
  u1(2) = v1(3)*v2(1) - v1(1)*v2(3)
  u1(3) = v1(1)*v2(2) - v1(2)*v2(1)
  
  u2(1) = v2(2)*v3(3) - v2(3)*v3(2)
  u2(2) = v2(3)*v3(1) - v2(1)*v3(3)
  u2(3) = v2(1)*v3(2) - v2(2)*v3(1)
  
  theta = dot_product(u1,u2)/(sqrt(dot_product(u1,u1))*sqrt(dot_product(u2,u2)))
  
  angle = acos(theta)
  
  if (angle .lt. 0.d0) then
    angle = angle + 6.2831853d0
  elseif (angle .gt. 6.2831853d0) then
    angle = 0.d0 !angle - 6.2831853d0  ! 0.d0 ?
  end if
  
  end subroutine dihed
  !ES-------------------------------------------------------
end module carint
!EM--------------------------
