!BM---------------------------------------
module forcefieldDS
 
contains
  
  !BF------------------------------------------------------
  function forceDS(x,syst,DSparam)
  use types;use assign_objects;use alloc_objects
  implicit none
  type(system_type),       intent(in)::syst
  type(potDS_type),       intent(in)::DSparam
  double precision, intent(in)::x(syst%Npart,3)
  character*2, allocatable, dimension(:)::atom_types
  double precision::g_mu(syst%Npart,3), g_nu(syst%Npart,3)
  double precision::forceDS(syst%Npart,3), rvec(3), rdist
  !double precision::force2(syst%Npart,3)
  double precision::DSqv(syst%Npart)
  double precision :: HH1, OH(7),OO(9), re, F(4), L(5)
  logical :: dipoleIter
  integer::N
  
  N=syst%Npart
  allocate(atom_types(N))
  atom_types=syst%ATOM_TYPES
  DSqv=DSparam%DSqv
  
  HH1=DSparam%DSHH1
  OH(1)=DSparam%DSOH1
  OH(2)=DSparam%DSOH2
  OH(3)=DSparam%DSOH3
  OH(4)=DSparam%DSOH4
  OH(5)=DSparam%DSOH5
  OH(6)=DSparam%DSOH6
  OH(7)=DSparam%DSOH7
  OO(1)=DSparam%DSOO1
  OO(2)=DSparam%DSOO2
  OO(3)=DSparam%DSOO3
  OO(4)=DSparam%DSOO4
  OO(5)=DSparam%DSOO5
  OO(6)=DSparam%DSOO6
  OO(7)=DSparam%DSOO7
  OO(8)=DSparam%DSOO8
  OO(9)=DSparam%DSOO9
  re=DSparam%DS_re
  F(1)=DSparam%DSF1
  F(2)=DSparam%DSF2
  F(3)=DSparam%DSF3
  F(4)=DSparam%DSF4
  L(1)=DSparam%DSL0
  L(2)=DSparam%DSL1
  L(3)=DSparam%DSL2
  L(4)=DSparam%DSL3
  L(5)=DSparam%DSL4
  dipoleIter=DSparam%DSdipole
  
  forceDS = 0.d0
  
  ! Calculating the pair-interaction forces
  forceDS(:,:) = F1(atom_types,x,N,re,HH1,OH,OO)
  
  ! Calculating the polrization forces from dipole moments
  if (dipoleIter) then
    call init_dipole(N,atom_types,x,DSqv,L,F,re,g_mu,g_nu)
    forceDS(:,:) = forceDS(:,:) + F2(atom_types,x,N,DSqv,g_mu,g_nu,re,F,L)
  end if
 
  !call writefunc(re,L,F,HH1,OH,OO)
  !stop

 
  end function forceDS
  !EF------------------------------------------------------
  
  subroutine writefunc(re,L,F,HH1,OH,OO)
  implicit none
  double precision, intent(in) :: re, L(5), F(4), HH1, OH(7),OO(9)
  double precision :: sk, sl, vhh, voh, voo, r
  integer :: i

  open(unit=333, file='screenKplot.out')
  open(unit=444, file='screenLplot.out')
  open(unit=555, file='HHplot.out')
  open(unit=666, file='OHplot.out')
  open(unit=777, file='OOplot.out')

  do i = 1,1000
      r = (0.d0 + i)/1000*3.5d0
      sk = s_K(r,re,F)
      write(333,*) r, sk
      voh =  v_OH(r,re,OH)
      write(666,*) r, voh
      vhh = v_HH(r,HH1)
      write(555,*) r, vhh
!--------------------------------------------------------!
      r = (0.d0 + i)/1000*7.d0
      sl = s_L(r,L)
      write(444,*) r, sl
      voo = v_OO(r,OO) 
      write(777,*) r, voo
  end do

  close(unit=333)
  close(unit=333)
  close(unit=333)
  close(unit=333)
  end subroutine writefunc

  !BF------------------------------------------------------
  function F1(atom_types,x,N,re,HH1,OH,OO)
  implicit none
  integer, intent(in)::N
  character*2, intent(in) :: atom_types(N)
  double precision, intent(in) :: re, HH1, OH(7),OO(9)
  double precision, intent(in) :: x(N,3)
  double precision :: r,f,F1(N,3),rvec(3),temp
  integer :: i,j,q
  
  F1 = 0.d0
  
  do i = 1,N-1
    do j = i+1,N
      rvec(:) = x(j,:) - x(i,:)
      r = sqrt(dot_product(rvec,rvec))
        if (atom_types(i) == "O" .and. atom_types(j)=="O") then
          f = f_OO(r,OO)/r
        elseif (atom_types(i) == "H" .and. atom_types(j)=="H") then
          f = f_HH(r,HH1)/r
        elseif (atom_types(i) == "O" .and. atom_types(j)=="H") then
          f = f_OH(r,re,OH)/r
        elseif (atom_types(i) == "H" .and. atom_types(j)=="O") then
          f = f_OH(r,re,OH)/r
        end if
        
        F1(i,:) = F1(i,:) + f*rvec(:)
        F1(j,:) = F1(j,:) - f*rvec(:)
        
    end do
  end do
  
  end function F1
  !EF------------------------------------------------------
  
  !BF------------------------------------------------------
  function f_HH(r,HH1)
  implicit none
  double precision, intent(in)::r,HH1
  double precision :: f_HH
  
  f_HH = -HH1/r**2
  
  end function f_HH
  !EF------------------------------------------------------
  
  !BF------------------------------------------------------
  function f_OH(r,re,OH) ! Model 78!
  implicit none
  double precision, intent(in)::r,re,OH(7)
  double precision :: f_OH, f1,f2,f3,f4,a,b,c,roh
  
  a = exp(OH(3)*r)
  f1 = (a*OH(1)*OH(4) + OH(1)*OH(2)*(-OH(3)*r-1))/(a*r**2)
  
  b = (exp(OH(7)*re*r))**2
  c = exp(OH(7)*re**2)*exp(OH(7)*r**2)
  f2 = b*OH(5)*((OH(7)*(2*re**2 - 4.*re*r + 2.*r**2)) - 1.)/c
  
  f3 = b*OH(6)*(OH(7)*(2*re**3-6.0*re**2*r+6.0*re*r**2-2.0*r**3)-2.0*re+2.0*r)/c
  
  f_OH = f1 + f2 + f3
  
  end function f_OH
  !EF------------------------------------------------------
  
  !BF------------------------------------------------------
  function f_OO(r,OO) ! Model 78!
  implicit none
  double precision, intent(in)::r,OO(9)
  double precision :: f_OO, a,b,f1,f2,f3,f4
  
  f1 = -OO(1)/r**2
  
  a = exp(OO(3)*OO(4))
  b = exp(OO(3)*r)
  f2 = -a*b*OO(2)*OO(3)/(a**2 + 2.0*a*b + b**2)
  
  a = exp(OO(6)*OO(7))
  b = exp(OO(6)*r)
  f3 = -a*b*OO(5)*OO(6)/(a**2 + 2.0*a*b + b**2)
  
  a = exp(OO(8)*OO(9))
  b = exp(OO(8)*r)
  f4 = -a*OO(8)/b
  
  !f4 = -OO(8)*exp(OO(8)*(OO(9)-r))  
  

  ! Alternatively, with squared exponent in fourth term of vOO:
  !a = exp(-OO(8)*(r-OO(9)))
  !a = exp(-OO(8)*(r-OO(9))**2)
  !b = r-OO(9)
  !f4 = 2.d0*OO(8)*(-b)*a
  
  f_OO = f1 + f2 + f3 + f4
  
  end function f_OO
  !EF------------------------------------------------------
  
  !BF------------------------------------------------------
  function F2(atom_types,x,N,DSqv,g_mu,g_nu,re,F,L)
  implicit none
  integer, intent(in)::N
  character*2, intent(in) :: atom_types(N)
  double precision, intent(in) :: x(N,3), DSqv(N)
  double precision, intent(in) :: g_mu(N,3), g_nu(N,3)
  double precision, intent(in) :: re, F(4), L(5)
  double precision :: F2(N,3)
  ! Parameters only used in calculation
  double precision :: mui(3), nui(3), muj(3), nuj(3)
  double precision :: qi, qj, rvec(3), r, ri, r3i
  double precision :: sl, sk, dl, dk
  double precision :: mir, mjr, nir, njr
  double precision :: Fm(3), Fn(3), Fmn(3), am, an
  double precision :: nimj, njmi, b, diTj(3), djTi(3)
  double precision, parameter :: aF = 0.5d0
  integer :: i, j 
  
  F2 = 0.d0
  
  do i = 1,N
    if (atom_types(i)=="O") then
      mui(:) = g_mu(i,:)  
      nui(:) = g_nu(i,:)
    end if
    
    
    do j = 1,N
      if (i/=j .and. (atom_types(j)=="O" .or. atom_types(i)=="O")) then
        if (atom_types(j)=="O") then
          muj(:) = g_mu(j,:)
          nuj(:) = g_nu(j,:)
        end if
        
        qi = DSqv(i)
        qj = DSqv(j)
        rvec = x(i,:) - x(j,:)
        r = sqrt(dot_product(rvec,rvec))
        ri = 1.d0/r
        r3i = ri**3
        
        sl = s_L(r,L) * r3i
        sk = s_K(r,re,F) * r3i
        dl = s_dL(r,L)*r3i - 3.d0*sl*ri
        dk = s_dK(r,re,F)*r3i - 3.d0*sk*ri
        
        mir = dot_product(mui,rvec)
        mjr = dot_product(muj,rvec)
        nir = dot_product(nui,rvec)
        njr = dot_product(nuj,rvec)
        
        if (atom_types(j)=="O" .and. atom_types(i)=="O") then
          Fm(:) = -aF * (muj(:)*qi - mui(:)*qj)*sl
          Fn(:) = -aF * (nuj(:)*qi - nui(:)*qj)*sk
          
          am = -aF*ri*(mjr*qi - mir*qj)*dl
          an = -aF*ri*(njr*qi - nir*qj)*dk
          
          Fm(:) = Fm(:) + am*rvec(:)
          Fn(:) = Fn(:) + an*rvec(:)
          
          nimj = dot_product(nui,muj)
          njmi = dot_product(nuj,mui)
          
          b = -0.5*ri*(njmi - 3.0*njr*mir*ri*ri + nimj - 3.0*mjr*nir*ri*ri)*dk
          
          djTi(:) = (nuj(:)*mir + mui(:)*njr)*r*r - 2.0*njr*mir*rvec(:)
          djTi(:) = djTi(:)*(-3.0*r3i*ri)
          diTj(:) = (nui(:)*mjr + muj(:)*nir)*r*r - 2.0*nir*mjr*rvec(:)
          diTj(:) = diTj(:)*(-3.0*r3i*ri)
          
          Fmn(:) = -0.5*(diTj(:) + djTi(:))*sk + b*rvec(:)
          
          F2(i,:) = F2(i,:) + Fm(:) + Fn(:) + Fmn(:)
        end if
        
        if (atom_types(j)=="H" .and. atom_types(i)=="O") then 
          am = aF * ri * mir * qj * dl
          an = aF * ri * nir * qj * dk
          
          Fm(:) = aF*mui(:)*qj*sl + am*rvec(:)
          Fn(:) = aF*nui(:)*qj*sk + an*rvec(:)
          
          F2(i,:) = F2(i,:) + Fm(:) + Fn(:)
          
        end if
        
        if (atom_types(j)=="O" .and. atom_types(i)=="H") then
          am = -aF*ri*mjr*qi*dl
          an = -aF*ri*njr*qi*dk
          
          Fm(:) = -aF*muj(:)*qi*sl + am*rvec(:)
          Fn(:) = -aF*nuj(:)*qi*sk + an*rvec(:)
          
          F2(i,:) = F2(i,:) + Fm(:) + Fn(:)
        end if
        
      end if
    end do
  end do
  
  end function F2
  !EF------------------------------------------------------
    
  !BF------------------------------------------------------  
  function EpotDS(x,syst,DSparam)
  use types;use assign_objects;use alloc_objects
  implicit none
  type(system_type),       intent(in)::syst
  type(potDS_type),        intent(in)::DSparam
  double precision, intent(in)::x(syst%Npart,3)
  double precision::EpotDS,rdist
  double precision::DSqv(syst%Npart)
  double precision::a,b
  character*2, allocatable, dimension(:)::atom_types
  double precision :: g_mu(syst%Npart,3),g_nu(syst%Npart,3)
  double precision :: g_mu2(syst%Npart,3)
  double precision :: HH1, OH(7),OO(10), re, F(4), L(5), thres
  logical :: dipoleIter
  integer::N
  
  N=syst%Npart
  allocate(atom_types(N))   
  atom_types=syst%ATOM_TYPES
  DSqv=DSparam%DSqv
  
  HH1=DSparam%DSHH1
  OH(1)=DSparam%DSOH1
  OH(2)=DSparam%DSOH2
  OH(3)=DSparam%DSOH3
  OH(4)=DSparam%DSOH4
  OH(5)=DSparam%DSOH5
  OH(6)=DSparam%DSOH6
  OH(7)=DSparam%DSOH7
  OO(1)=DSparam%DSOO1
  OO(2)=DSparam%DSOO2
  OO(3)=DSparam%DSOO3
  OO(4)=DSparam%DSOO4
  OO(5)=DSparam%DSOO5
  OO(6)=DSparam%DSOO6
  OO(7)=DSparam%DSOO7
  OO(8)=DSparam%DSOO8
  OO(9)=DSparam%DSOO9
  re=DSparam%DS_re
  F(1)=DSparam%DSF1
  F(2)=DSparam%DSF2
  F(3)=DSparam%DSF3
  F(4)=DSparam%DSF4
  L(1)=DSparam%DSL0
  L(2)=DSparam%DSL1
  L(3)=DSparam%DSL2
  L(4)=DSparam%DSL3
  L(5)=DSparam%DSL4
  dipoleIter=DSparam%DSdipole
  
  EpotDS = 0.d0
  
  ! Pair interaction energy
  EpotDS = psi1(atom_types,x,N,re,HH1,OH,OO)
  
  ! Polarization energy from dipole moments
  if (dipoleIter) then
    call init_dipole(N,atom_types,x,DSqv,L,F,re,g_mu,g_nu)
    EpotDS = EpotDS + psi2(atom_types,x,N,DSqv,L,g_mu)
  end if
  
  end function EpotDS
  !EF------------------------------------------------------
  
  !BF------------------------------------------------------
  function psi1(atom_types,x,N,re,HH1,OH,OO)
  implicit none
  integer, intent(in)::N
  character*2, intent(in) :: atom_types(N)
  double precision, intent(in) :: HH1, OH(7), OO(10)
  double precision, intent(in) :: x(N,3)
  double precision :: rvec(3),r,re, psi1
  integer :: i,j
  
  psi1 = 0.d0
  
  do i=1,N-1
    do j=i+1,N
        rvec(:) = x(j,:) - x(i,:)
        r = sqrt(dot_product(rvec,rvec))
        if (atom_types(i) == "O" .and. atom_types(j)=="O") then
          psi1 = psi1 + v_OO(r,OO)
        elseif (atom_types(i) == "H" .and. atom_types(j)=="H") then
          psi1 = psi1 + v_HH(r,HH1)
        elseif (atom_types(i) == "O" .and. atom_types(j)=="H") then
          psi1 = psi1 + v_OH(r,re,OH)
        elseif (atom_types(i) == "H" .and. atom_types(j)=="O") then
          psi1 = psi1 + v_OH(r,re,OH)
        end if
    end do
  end do
  
  end function psi1
  !EF------------------------------------------------------
  
  !BF------------------------------------------------------
  function v_HH(r,HH1)
  implicit none
  double precision, intent(in)::r,HH1
  double precision :: v_HH
  
  v_HH = HH1/r
  
  end function v_HH
  !EF------------------------------------------------------
  
  !BF------------------------------------------------------
  function v_OH(r,re,OH) ! Model 78!
  implicit none
  double precision, intent(in)::r,re,OH(7)
  double precision :: v_OH
  double precision :: roh
  roh=r-re
  
  v_OH = (OH(1)/r)*(OH(2)*exp(-OH(3)*r)-OH(4)) + &
         (-OH(5)*roh + OH(6)*roh**2)*exp(-OH(7)*roh**2)
  
  end function v_OH
  !EF------------------------------------------------------
  
  !BF------------------------------------------------------
  function v_OO(r,OO) ! Model 78!
  implicit none
  double precision, intent(in)::r,OO(9)
  double precision :: v_OO
  
  ! From original article
  v_OO = OO(1)/r + &
         OO(2)/(1+exp(OO(3)*(r-OO(4)))) + &
         OO(5)/(1+exp(OO(6)*(r-OO(7)))) + &
         exp(-OO(8)*(r-OO(9)))
  
  ! From Dellago-code
  !v_OO = OO(1)/r + &
  !       OO(2)/(1+exp(OO(3)*(r-OO(4)))) + &
  !       OO(5)/(1+exp(OO(6)*(r-OO(7)))) + &
  !       exp(-OO(8)*(r-OO(9))**2)
  
  ! NB! Use correct derivative in function f_OO
  
  end function v_OO
  !EF------------------------------------------------------
  
  !BF------------------------------------------------------
  function psi2(atom_types,x,N,DSqv,L,g_mu)
  implicit none
  integer, intent(in)::N
  character*2, intent(in) :: atom_types(N)
  double precision, intent(in) :: x(N,3), DSqv(N)
  double precision, intent(in) :: L(5), g_mu(N,3)
  double precision :: psi2, roh, rvec(3), rdist, mui(3), mq, sl
  integer :: i,j
  
  psi2 = 0.d0
  
  do i = 1,N
    if (atom_types(i)=="H") cycle
      mui(:) = g_mu(i,:)
      do j = 1,N
        if (i/=j .and. (atom_types(j)=="O" .or. atom_types(i)=="O")) then
          rvec = x(j,:) - x(i,:)
          rdist = sqrt(dot_product(rvec,rvec))
          mq = dot_product(mui,rvec)
          sl = s_L(rdist,L)/(rdist**3)
          psi2 = psi2 + 0.5*mq*sl*DSqv(j)
        end if !
      end do
  end do  
  
  end function psi2
  !EF------------------------------------------------------
  
  !BF------------------------------------------------------
  function s_K(r,re,F)
  implicit none
  double precision, intent(in) :: r,re,F(4)
  double precision :: s_K
  
  s_K = r**3/(r**3 + s_F(r,re,F))
  
  end function s_K 
  !EF------------------------------------------------------
  
  !BF------------------------------------------------------
  function s_dK(r,re,F)
  implicit none
  double precision, intent(in) :: r,re,F(4)
  double precision :: s_dK, dF, u
  
  u = r**3 + s_F(r,re,F)
  dF = s_dF(r,re,F)
  s_dK = (3*r**2*u - r**3*(3*r**2 + dF))/u**2
  
  end function s_dK
  !EF------------------------------------------------------
  
  !BF------------------------------------------------------
  function s_F(r,re,F)
  implicit none
  double precision, intent(in) :: r,re,F(4)
  double precision :: s_F, x
  
  x = (r-re)**2
  
  s_F = F(1)*x*exp(-F(2)*x) + F(3)*exp(-F(4)*r)
  
  end function s_F 
  !EF------------------------------------------------------
  
  !BF------------------------------------------------------
  function s_dF(r,re,F)
  implicit none
  double precision, intent(in) :: r,re,F(4)
  double precision :: s_dF, rr
  
  rr = r - re
  
  s_dF = exp(-F(2)*rr**2)*(2*F(1)*rr - 2*F(1)*F(2)*rr**3) - F(3)*F(4)*exp(-F(4)*r)
  
  end function s_dF
  !EF------------------------------------------------------
  
  !BF------------------------------------------------------
  function s_L(r,L)
  implicit none
  double precision, intent(in) :: r, L(5)
  double precision :: s_L
  
  s_L = 1.d0 - exp(-L(1)*r)*(1.d0 + L(2)*r + L(3)*r**2 - L(4)*r**3 + L(5)*r**4)
  
  end function s_L
  !EF------------------------------------------------------
  
  !BF------------------------------------------------------
  function s_dL(r,L)
  implicit none
  double precision, intent(in) :: r, L(5)
  double precision :: s_dL, sl
  double precision :: r2, r3, r4, e 
  
  r2 = r**2
  r3 = r2*r
  r4 = r3*r
  
  e = exp(-L(1)*r)
  sl = e*(1.d0 + L(2)*r + L(3)*r2 - L(4)*r3 + L(5)*r4)
  s_dl = -L(2)*sl + e*(L(2) + 2.d0*L(3)*r - 3.d0*L(4)*r2 + 4.d0*L(5)*r3)
  s_dl = -s_dl
  
  end function s_dL
  !EF------------------------------------------------------
  
  !BS------------------------------------------------------
  subroutine init_dipole(N,atom_types,x,DSqv,L,F,re,g_mu,g_nu)
  implicit none
  integer, intent(in)::N
  character*2, intent(in) :: atom_types(N)
  double precision, intent(in) :: x(N,3)
  double precision, intent(in) :: DSqv(N), L(5), F(4), re
  double precision :: g_mu(N,3), g_nu(N,3)
  double precision :: tolerance
  integer :: iter_max
  
  g_mu = 0.d0
  g_nu = 0.d0
  iter_max = 100
  tolerance = 1e-18
  call iterate_dipole(N,atom_types,x,DSqv,L,F,re,g_mu,g_nu,iter_max,tolerance)
  
  end subroutine init_dipole
  !ES------------------------------------------------------
  
  !BS------------------------------------------------------
  subroutine iterate_dipole(N,atom_types,x,DSqv,L,F,re,mu,nu,iter_max,tolerance)
  implicit none
  integer, intent(in)::N
  character*2, intent(in) :: atom_types(N)
  double precision, intent(in) :: x(N,3)
  double precision, intent(in) :: DSqv(N), L(5), F(4), re
  double precision, intent(inout) :: mu(N,3), nu(N,3)
  double precision, intent(in) :: tolerance
  integer, intent(in) :: iter_max
  double precision :: A(3*N,3*N), diff
  double precision :: c_mu(N,3), c_nu(N,3)
  double precision :: mu_old(N,3), nu_old(N,3)
  !double precision :: mu(N,3), nu(N,3)
  integer :: niter
  
  niter = 0
  diff = 10*tolerance ! To make sure the while-loop doesn't exit at first run
  call get_matrix(N,atom_types,x,DSqv,re,L,F,A,c_mu,c_nu)
  
  do while (diff > tolerance)
  
  mu_old = mu
  nu_old = nu
  
  call get_dipole(N, A, c_mu, c_nu, mu_old, mu, nu_old, nu)
  
  diff = dipole_acc(N, mu, mu_old, nu, nu_old)
  
  niter = niter + 1
  
  if (niter > iter_max) then
    print*, "Dipole_iter max, diff:",diff,", tolerance:",tolerance
    exit
  end if 
  
  end do
  
  end subroutine iterate_dipole
  !ES------------------------------------------------------
  
  !BS------------------------------------------------------
  subroutine get_matrix(N,atom_types,x,DSqv,re,Lr,Fr,A,c_mu,c_nu)
  implicit none
  integer, intent(in)::N
  character*2, intent(in) :: atom_types(N)
  double precision, intent(in) :: x(N,3)
  double precision, intent(in) :: DSqv(N), re, Lr(5), Fr(4)
  double precision :: A(3*N,3*N)
  double precision :: c_mu(N,3), c_nu(N,3)
  double precision :: rvec(3), r, r3i, r5i
  double precision :: am, b, sK, sL
  double precision, parameter :: pol = 1.444
  integer :: i,j,l,m,il,jm
  
  !
  ! Important!
  ! Change struct of A to a 3-D array instead - less zeroes!
  ! --> only less if size equals number of oxygens...
  !       ...hmm....
  !
  
  A = 0.d0
  c_mu = 0.d0
  c_nu = 0.d0
  
  do i = 1,N
    if (atom_types(i) == "H") cycle
      do j = 1,N
        if (i==j) cycle
          rvec(:) = x(j,:) - x(i,:)
          r = sqrt(dot_product(rvec,rvec))
          r3i = 1.d0/(r**3)
          r5i = r3i/(r**2)
          
          sK = s_K(r,re,Fr)*pol
          sL = s_L(r,Lr)*pol
          
          am = DSqv(j)*r3i*sK
          b = DSqv(j)*r3i*sL
          c_mu(i,:) = c_mu(i,:) - am*rvec(:)
          c_nu(i,:) = c_nu(i,:) - b*rvec(:)
          
        if (atom_types(j) == "O") then
          m = i - 1
          l = j - 1
          am = r3i*sK
          b = 3.d0*r5i*sK
          
          A(m*3+1,l*3+1) = A(m*3+1,l*3+1) - am
          A(m*3+2,l*3+2) = A(m*3+2,l*3+2) - am
          A(m*3+3,l*3+3) = A(m*3+3,l*3+3) - am
          
          A(m*3+1,l*3+1) = A(m*3+1,l*3+1) + b*rvec(1)*rvec(1)
          A(m*3+2,l*3+1) = A(m*3+2,l*3+1) + b*rvec(2)*rvec(1)
          A(m*3+3,l*3+1) = A(m*3+3,l*3+1) + b*rvec(3)*rvec(1)
          
          A(m*3+1,l*3+2) = A(m*3+1,l*3+2) + b*rvec(1)*rvec(2)
          A(m*3+2,l*3+2) = A(m*3+2,l*3+2) + b*rvec(2)*rvec(2)
          A(m*3+3,l*3+2) = A(m*3+3,l*3+2) + b*rvec(3)*rvec(2)
          
          A(m*3+1,l*3+3) = A(m*3+1,l*3+3) + b*rvec(1)*rvec(3)
          A(m*3+2,l*3+3) = A(m*3+2,l*3+3) + b*rvec(2)*rvec(3)
          A(m*3+3,l*3+3) = A(m*3+3,l*3+3) + b*rvec(3)*rvec(3)
          
        end if
        
      end do
      
  end do
  
  end subroutine get_matrix
  !ES------------------------------------------------------
  
  !BS------------------------------------------------------
  subroutine get_dipole(N, A, c_mu, c_nu, mu_old, mu_new, nu_old, nu_new)
  implicit none
  integer, intent(in) :: N
  double precision, intent(in) :: A(3*N,3*N)
  double precision, intent(in) :: c_mu(N,3), c_nu(N,3)
  double precision, intent(in) :: mu_old(N,3), nu_old(N,3)
  double precision :: mu_new(N,3), nu_new(N,3), tempA(3,3)
  integer :: i,j,k,l,m,o
  
  do i = 1,N
    mu_new(i,:) = c_mu(i,:)
    nu_new(i,:) = c_nu(i,:)
    do j = 1,N
      do l = 1,3
        do k = 1,3
          o = 3*j + k - 3
          m = 3*i + l - 3
      
          mu_new(i,l) = mu_new(i,l) + A(m,o)*mu_old(j,k)
          nu_new(i,l) = nu_new(i,l) + A(m,o)*nu_old(j,k)
          
        end do
      end do
    end do 
  end do 
  
  end subroutine get_dipole
  !ES------------------------------------------------------
  
  !BF------------------------------------------------------
  function dipole_acc(N, mu, mu_old, nu, nu_old)
  implicit none
  integer, intent(in) :: N
  double precision, intent(in) :: mu(N,3), nu(N,3)
  double precision, intent(in) :: mu_old(N,3), nu_old(N,3)
  double precision :: dm, m, dipole_acc
  integer :: i,j
  
  dm = 0.d0
  m = 0.d0
  
  do i = 1,N
    do j = 1,3
      dm = dm + (mu(i,j) - mu_old(i,j))**2
      dm = dm + (nu(i,j) - nu_old(i,j))**2
      m = m + (mu(i,j)**2 + mu_old(i,j)**2) * 0.5
      m = m + (nu(i,j)**2 + nu_old(i,j)**2) * 0.5
    end do
  end do
  
  dipole_acc = dm/m
  
  end function
  !EF------------------------------------------------------
  
end module forcefieldDS
!EM---------------------------------------
