!BM---------------------------------------
module forcefieldqchem

contains
  !BS-------------------------------------------
  subroutine execQCHEM(x,syst)
  use stringlengths;use shell
  use types;use assign_objects;use alloc_objects
  implicit none
  type(system_type), intent(in) :: syst
  integer :: N, i, ncpu, counter
  character*2 :: atom_types(syst%Npart)
  double precision, intent(in) :: x(syst%Npart,3)
  logical :: waiting
  !character(LEN=LSTR)::command
  
  ncpu = syst%ncpu
  N = syst%Npart
  atom_types=syst%ATOM_TYPES 
  
  ! This part of the program creates a geometry file that can be used by the
  ! QCHEM input files. Cartesian coordinates and atom types are written to a
  ! file with the required start and end, see below.
  
  open(unit=666,file='QGEOMETRY')
  write(666,'(a9)') '$molecule'
  write(666,'(a6)') '   1 1' 
! The two numbers above are the charge of the system and the multiplicity
  do i = 1,N
     write(666,'(A4,3F11.6)') atom_types(i), x(i,:)
  end do
  write(666,'(a4)') '$end'
  close(unit=666)
 
  ! After creating th QGEOMETRY file QChem are called to get the energy and
  ! forces.

  call do_shell('./run_qchem.sh')

  counter=0
  do
    INQUIRE(file='QCHEM.OUT', EXIST=waiting)
    if(waiting) go to 30
    call sleep(1)
    counter=counter+1
    if(counter.gt.1000)then
      write(*,*)'Waited 1000 seconds..'
      write(*,*)'QCHEM crashed..?'
      stop
    end if
  end do
30 continue

  write(*,*) 'Waited', counter, 'seconds on QChem'
  call do_shell('rm QGEOMETRY')
  
  end subroutine execQCHEM
  !ES-------------------------------------------
  
  !BF-------------------------------------------
  function forceQCHEM(x,syst)
  use shell; use stringlengths
  use types;use assign_objects;use alloc_objects
  implicit none
  type(system_type),     intent(in)::syst
  double precision, intent(in)::x(syst%Npart,syst%dim)
  character*2 :: atom_types(syst%Npart)
  double precision::forceQCHEM(syst%Npart,syst%dim),meh
  character(LEN=52) ::grdkey 
  logical :: filecheck

  ! The function checks if the output from QCHEM exist, if not it will call the
  ! executing subroutine to create one and read from it. If the output exist it
  ! will read it, and see if the energy function has already read it. If so, it
  ! will delete it. The file QFcheck is a empty file created as a flag that the
  ! forces have been read from the file.
  
  !grdkey="Full Analytical Gradient: (in a.u., or Hartree/Bohr)"
  !grdkey="Nuclear "
  grdkey="Full "

  INQUIRE(file='QFcheck', EXIST=filecheck)
  if (filecheck) then
    call do_shell('rm -rf QCHEM.OUT QFcheck QEcheck')
    call execQCHEM(x,syst)
  end if
 
  INQUIRE(file= 'QCHEM.OUT', EXIST=filecheck)
  if (.not. filecheck) then
    call execQCHEM(x,syst)
    call read_gradient(syst,forceQCHEM,'QCHEM.OUT',grdkey)
    call do_shell('touch QFcheck')
  else
    call read_gradient(syst,forceQCHEM,'QCHEM.OUT',grdkey)
    call do_shell('touch QFcheck')
      INQUIRE(file= 'QEcheck', EXIST=filecheck)
      if (filecheck) then
        call do_shell('rm -rf QCHEM.OUT QEcheck')
      end if
  end if
 
  end function forceQCHEM
  !EF------------------------------------------


  !BF------------------------------------------
  function EpotQCHEM(x,syst)
  use read_input_file; use stringlengths
  use types;use assign_objects;use alloc_objects
  use shell
  implicit none
  type(system_type),       intent(in)::syst
  double precision, intent(in)::x(syst%Npart,syst%dim)
  character*2 :: atom_types(syst%Npart)
  character(LEN=31)::enkey
  double precision::EpotQCHEM
  logical :: filecheck

  ! The function checks if the output from QCHEM exist, if not it will call the
  ! executing subroutine to create one and read from it. If the output exist it
  ! will read it, and see if the force function has already read it. If so, it
  ! will delete it. The file QEcheck is a empty file created as a flag that the
  ! energy have been read from the file.

  enkey=' CCSD total energy           ='

  INQUIRE(file='QEcheck', EXIST=filecheck)
  if (filecheck) then
    call do_shell('rm -rf QCHEM.OUT QFcheck QEcheck')
    call execQCHEM(x,syst)
  end if
 
  INQUIRE(file= 'QCHEM.OUT', EXIST=filecheck)
  if (.not. filecheck) then
    call execQCHEM(x,syst)
    call read_energy(EpotQCHEM,'QCHEM.OUT',enkey)
    call do_shell('touch QEcheck')
  else
    call read_energy(EpotQCHEM,'QCHEM.OUT',enkey)
      INQUIRE(file= 'QFcheck', EXIST=filecheck)
      if (filecheck) then
        call do_shell('rm -rf QCHEM.OUT QEcheck')
      end if
  end if

  end function EpotQCHEM
  !EF------------------------------------------

  !BS------------------------------------------
  subroutine read_energy(en,filenm,keyword)
  implicit none
  character(LEN=9),   intent(in)::filenm
  double precision::en
  character(LEN=*)::keyword
  character(LEN=LEN(keyword)-1)::word
  integer::status
  logical::keywordfound

  keywordfound=.false.
  open(1,file=filenm,status="old")
    do
      
      read(1,'(a)',iostat=status) word
      if (status /=0) exit
      if (word==keyword) then
        keywordfound=.true.

        backspace 1
        read(1,'(a,f14.8)') word, en  
      endif

      if (keywordfound) exit

    enddo
  close(1)

  if (.not. keywordfound) then
    print*, "ERROR"
    print*, "Could not find keyword for energy in ",filenm," that match:"
    print*, keyword
    stop
  endif

  end subroutine read_energy
  !ES------------------------------------------

  !BS------------------------------------------
  subroutine read_gradient(syst,grd,filenm,keyword)
  use types;use assign_objects;use alloc_objects
  implicit none
  type(system_type),     intent(in)::syst
  character(LEN=9),   intent(in)::filenm
  double precision::grd(syst%Npart,syst%dim)
  character(LEN=*)::keyword
  character(LEN=LEN(keyword)+1)::word
  integer::status, k,i,j,N,Na,b,c(5)
  logical::keywordfound

  N = syst%Npart
  Na = N/5

  keywordfound=.false.
  open(1,file=filenm,status="old")
    do
      
      read(1,*,iostat=status) word
      if (status /=0) exit
      if (word==keyword) then
        keywordfound=.true.

        do i = 1,Na
          j = (i-1)*5
          read(1,*,iostat=status) c(:)
          read(1,'(i5,5f14.8)',iostat=status) b,grd(j+1:j+5,1)
          read(1,'(i5,5f14.8)',iostat=status) b,grd(j+1:j+5,2)
          read(1,'(i5,5f14.8)',iostat=status) b,grd(j+1:j+5,3)
          if (status /=0) then
            print *, "ERROR: read-gradient"
            stop
          endif
        enddo
      endif

      if (keywordfound) exit

    enddo
  close(1)

  if (.not. keywordfound) then
    print*, "ERROR"
    print*, "Could not find keyword for gradient in ",filenm," that match:"
    print*, keyword
    stop
  endif

  !do i = 1,10
  !  write(*,*) i, grd(i,:)
  !end do

  !stop

  ! Reversing the sign of the gradient and converting from Bohr to Angstrom, REQUIRED!

  grd(:,:) = grd(:,:)/-0.529177211

  end subroutine read_gradient

end module forcefieldqchem
!EM---------------------------------------

